from enum import Enum
import asyncio
from aadb import events
from aadb.adb import AndroidDebugBridge


class KeyCode(Enum):
    KEYCODE_UNKNOWN = 0
    KEYCODE_SOFT_LEFT = 1
    KEYCODE_SOFT_RIGHT = 2
    KEYCODE_HOME = 3
    KEYCODE_BACK = 4
    KEYCODE_CALL = 5
    KEYCODE_ENDCALL = 6
    KEYCODE_0 = 7
    KEYCODE_1 = 8
    KEYCODE_2 = 9
    KEYCODE_3 = 10
    KEYCODE_4 = 11
    KEYCODE_5 = 12
    KEYCODE_6 = 13
    KEYCODE_7 = 14
    KEYCODE_8 = 15
    KEYCODE_9 = 16
    KEYCODE_STAR = 17
    KEYCODE_POUND = 18
    KEYCODE_DPAD_UP = 19
    KEYCODE_DPAD_DOWN = 20
    KEYCODE_DPAD_LEFT = 21
    KEYCODE_DPAD_RIGHT = 22
    KEYCODE_DPAD_CENTER = 23
    KEYCODE_VOLUME_UP = 24
    KEYCODE_VOLUME_DOWN = 25
    KEYCODE_POWER = 26
    KEYCODE_CAMERA = 27
    KEYCODE_CLEAR = 28
    KEYCODE_A = 29
    KEYCODE_B = 30
    KEYCODE_C = 31
    KEYCODE_D = 32
    KEYCODE_E = 33
    KEYCODE_F = 34
    KEYCODE_G = 35
    KEYCODE_H = 36
    KEYCODE_I = 37
    KEYCODE_J = 38
    KEYCODE_K = 39
    KEYCODE_L = 40
    KEYCODE_M = 41
    KEYCODE_N = 42
    KEYCODE_O = 43
    KEYCODE_P = 44
    KEYCODE_Q = 45
    KEYCODE_R = 46
    KEYCODE_S = 47
    KEYCODE_T = 48
    KEYCODE_U = 49
    KEYCODE_V = 50
    KEYCODE_W = 51
    KEYCODE_X = 52
    KEYCODE_Y = 53
    KEYCODE_Z = 54
    KEYCODE_COMMA = 55
    KEYCODE_PERIOD = 56
    KEYCODE_ALT_LEFT = 57
    KEYCODE_ALT_RIGHT = 58
    KEYCODE_SHIFT_LEFT = 59
    KEYCODE_SHIFT_RIGHT = 60
    KEYCODE_TAB = 61
    KEYCODE_SPACE = 62
    KEYCODE_SYM = 63
    KEYCODE_EXPLORER = 64
    KEYCODE_ENVELOPE = 65
    KEYCODE_ENTER = 66
    KEYCODE_DEL = 67
    KEYCODE_GRAVE = 68
    KEYCODE_MINUS = 69
    KEYCODE_EQUALS = 70
    KEYCODE_LEFT_BRACKET = 71
    KEYCODE_RIGHT_BRACKET = 72
    KEYCODE_BACKSLASH = 73
    KEYCODE_SEMICOLON = 74
    KEYCODE_APOSTROPHE = 75
    KEYCODE_SLASH = 76
    KEYCODE_AT = 77
    KEYCODE_NUM = 78
    KEYCODE_HEADSETHOOK = 79
    KEYCODE_FOCUS = 80
    KEYCODE_PLUS = 81
    KEYCODE_MENU = 82
    KEYCODE_NOTIFICATION = 83
    KEYCODE_SEARCH = 84
    KEYCODE_MEDIA_PLAY_PAUSE = 85
    KEYCODE_MEDIA_STOP = 86
    KEYCODE_MEDIA_NEXT = 87
    KEYCODE_MEDIA_PREVIOUS = 88
    KEYCODE_MEDIA_REWIND = 89
    KEYCODE_MEDIA_FAST_FORWARD = 90
    KEYCODE_MUTE = 91
    KEYCODE_PAGE_UP = 92
    KEYCODE_PAGE_DOWN = 93
    KEYCODE_PICTSYMBOLS = 94
    KEYCODE_SWITCH_CHARSET = 95
    KEYCODE_BUTTON_A = 96
    KEYCODE_BUTTON_B = 97
    KEYCODE_BUTTON_C = 98
    KEYCODE_BUTTON_X = 99
    KEYCODE_BUTTON_Y = 100
    KEYCODE_BUTTON_Z = 101
    KEYCODE_BUTTON_L1 = 102
    KEYCODE_BUTTON_R1 = 103
    KEYCODE_BUTTON_L2 = 104
    KEYCODE_BUTTON_R2 = 105
    KEYCODE_BUTTON_THUMBL = 106
    KEYCODE_BUTTON_THUMBR = 107
    KEYCODE_BUTTON_START = 108
    KEYCODE_BUTTON_SELECT = 109
    KEYCODE_BUTTON_MODE = 110
    KEYCODE_ESCAPE = 111
    KEYCODE_FORWARD_DEL = 112
    KEYCODE_CTRL_LEFT = 113
    KEYCODE_CTRL_RIGHT = 114
    KEYCODE_CAPS_LOCK = 115
    KEYCODE_SCROLL_LOCK = 116
    KEYCODE_META_LEFT = 117
    KEYCODE_META_RIGHT = 118
    KEYCODE_FUNCTION = 119
    KEYCODE_SYSRQ = 120
    KEYCODE_BREAK = 121
    KEYCODE_MOVE_HOME = 122
    KEYCODE_MOVE_END = 123
    KEYCODE_INSERT = 124
    KEYCODE_FORWARD = 125
    KEYCODE_MEDIA_PLAY = 126
    KEYCODE_MEDIA_PAUSE = 127
    KEYCODE_MEDIA_CLOSE = 128
    KEYCODE_MEDIA_EJECT = 129
    KEYCODE_MEDIA_RECORD = 130
    KEYCODE_F1 = 131
    KEYCODE_F2 = 132
    KEYCODE_F3 = 133
    KEYCODE_F4 = 134
    KEYCODE_F5 = 135
    KEYCODE_F6 = 136
    KEYCODE_F7 = 137
    KEYCODE_F8 = 138
    KEYCODE_F9 = 139
    KEYCODE_F10 = 140
    KEYCODE_F11 = 141
    KEYCODE_F12 = 142
    KEYCODE_NUM_LOCK = 143
    KEYCODE_NUMPAD_0 = 144
    KEYCODE_NUMPAD_1 = 145
    KEYCODE_NUMPAD_2 = 146
    KEYCODE_NUMPAD_3 = 147
    KEYCODE_NUMPAD_4 = 148
    KEYCODE_NUMPAD_5 = 149
    KEYCODE_NUMPAD_6 = 150
    KEYCODE_NUMPAD_7 = 151
    KEYCODE_NUMPAD_8 = 152
    KEYCODE_NUMPAD_9 = 153
    KEYCODE_NUMPAD_DIVIDE = 154
    KEYCODE_NUMPAD_MULTIPLY = 155
    KEYCODE_NUMPAD_SUBTRACT = 156
    KEYCODE_NUMPAD_ADD = 157
    KEYCODE_NUMPAD_DOT = 158
    KEYCODE_NUMPAD_COMMA = 159
    KEYCODE_NUMPAD_ENTER = 160
    KEYCODE_NUMPAD_EQUALS = 161
    KEYCODE_NUMPAD_LEFT_PAREN = 162
    KEYCODE_NUMPAD_RIGHT_PAREN = 163
    KEYCODE_VOLUME_MUTE = 164
    KEYCODE_INFO = 165
    KEYCODE_CHANNEL_UP = 166
    KEYCODE_CHANNEL_DOWN = 167
    KEYCODE_ZOOM_IN = 168
    KEYCODE_ZOOM_OUT = 169
    KEYCODE_TV = 170
    KEYCODE_WINDOW = 171
    KEYCODE_GUIDE = 172
    KEYCODE_DVR = 173
    KEYCODE_BOOKMARK = 174
    KEYCODE_CAPTIONS = 175
    KEYCODE_SETTINGS = 176
    KEYCODE_TV_POWER = 177
    KEYCODE_TV_INPUT = 178
    KEYCODE_STB_POWER = 179
    KEYCODE_STB_INPUT = 180
    KEYCODE_AVR_POWER = 181
    KEYCODE_AVR_INPUT = 182
    KEYCODE_PROG_RED = 183
    KEYCODE_PROG_GREEN = 184
    KEYCODE_PROG_YELLOW = 185
    KEYCODE_PROG_BLUE = 186
    KEYCODE_APP_SWITCH = 187
    KEYCODE_BUTTON_1 = 188
    KEYCODE_BUTTON_2 = 189
    KEYCODE_BUTTON_3 = 190
    KEYCODE_BUTTON_4 = 191
    KEYCODE_BUTTON_5 = 192
    KEYCODE_BUTTON_6 = 193
    KEYCODE_BUTTON_7 = 194
    KEYCODE_BUTTON_8 = 195
    KEYCODE_BUTTON_9 = 196
    KEYCODE_BUTTON_10 = 197
    KEYCODE_BUTTON_11 = 198
    KEYCODE_BUTTON_12 = 199
    KEYCODE_BUTTON_13 = 200
    KEYCODE_BUTTON_14 = 201
    KEYCODE_BUTTON_15 = 202
    KEYCODE_BUTTON_16 = 203
    KEYCODE_LANGUAGE_SWITCH = 204
    KEYCODE_MANNER_MODE = 205
    KEYCODE_3D_MODE = 206
    KEYCODE_CONTACTS = 207
    KEYCODE_CALENDAR = 208
    KEYCODE_MUSIC = 209
    KEYCODE_CALCULATOR = 210
    KEYCODE_ZENKAKU_HANKAKU = 211
    KEYCODE_EISU = 212
    KEYCODE_MUHENKAN = 213
    KEYCODE_HENKAN = 214
    KEYCODE_KATAKANA_HIRAGANA = 215
    KEYCODE_YEN = 216
    KEYCODE_RO = 217
    KEYCODE_KANA = 218
    KEYCODE_ASSIST = 219
    KEYCODE_BRIGHTNESS_DOWN = 220
    KEYCODE_BRIGHTNESS_UP = 221
    KEYCODE_MEDIA_AUDIO_TRACK = 222


inner_host = ''
inner_port = ''


def create_bridge(host: str = '127.0.0.1', port: int = 5037):
    global inner_port
    global inner_host
    inner_host = host
    inner_port = port
    return AndroidDebugBridge()


def start(func):
    events.set_event_loop(asyncio.get_event_loop())
    events.get_event_loop().run_until_complete(func)
    events.get_event_loop().run_forever()



import asyncio
import threading
from asyncio import AbstractEventLoop

_lock = threading.Lock()

_event_loop_policy: 'DefaultPolicy' = None


class DefaultPolicy(object):
    class _Local(object):
        _loop: AbstractEventLoop = None
        _called = False

    def set_event_loop(self, loop: AbstractEventLoop):
        self._Local._called = True
        self._Local._loop = loop

    def get_event_loop(self) -> AbstractEventLoop:
        if self._Local._loop is None and not self._Local._called:
            self.set_event_loop(asyncio.new_event_loop() if threading.current_thread() != threading.main_thread()
                                else asyncio.get_event_loop())
        return self._Local._loop


def _init_event_loop_policy():
    global _event_loop_policy
    with _lock:
        if _event_loop_policy is None:
            _event_loop_policy = DefaultPolicy()


def get_event_loop_policy() -> DefaultPolicy:
    if _event_loop_policy is None:
        _init_event_loop_policy()
    return _event_loop_policy


def get_event_loop() -> AbstractEventLoop:
    return get_event_loop_policy().get_event_loop()


def set_event_loop(loop: AbstractEventLoop):
    get_event_loop_policy().set_event_loop(loop)



import aadb
import asyncio


async def main():
    adb = aadb.create_bridge()
    device = (await adb.devices())[0]
    asyncio.create_task(device.logcat(pipeline=lambda x: print(x)))
    await asyncio.sleep(3)
    print(await device.get_properties())
    await asyncio.sleep(3)
    print(await device.list_package())
    await device.shell('dumpsys meminfo', pipeline=lambda x: print(x))

if __name__ == '__main__':
    aadb.start(main())



import asyncio
from asyncio import StreamReader, StreamWriter
from asyncio import StreamReaderProtocol

import aadb
from aadb import events


class Signal(object):
    OKAY = 'OKAY'
    FAIL = 'FAIL'
    STAT = 'STAT'
    LIST = 'LIST'
    DENT = 'DENT'
    RECV = 'RECV'
    DATA = 'DATA'
    DONE = 'DONE'
    SEND = 'SEND'
    QUIT = 'QUIT'


class Stats(object):
    # The following constant were extracted from `man 2 stat` on Ubuntu 12.10.
    S_IFMT = 0o170000  # bit mask for the file type bit fields
    S_IFSOCK = 0o140000  # socket
    S_IFLNK = 0o120000  # symbolic link
    S_IFREG = 0o100000  # regular file
    S_IFBLK = 0o060000  # block device
    S_IFDIR = 0o040000  # directory
    S_IFCHR = 0o020000  # character device
    S_IFIFO = 0o010000  # FIFO
    S_ISUID = 0o004000  # set UID bit
    S_ISGID = 0o002000  # set-group-ID bit (see below)
    S_ISVTX = 0o001000  # sticky bit (see below)
    S_IRWXU = 0o0700  # mask for file owner permissions
    S_IRUSR = 0o0400  # owner has read permission
    S_IWUSR = 0o0200  # owner has write permission
    S_IXUSR = 0o0100  # owner has execute permission
    S_IRWXG = 0o0070  # mask for group permissions
    S_IRGRP = 0o0040  # group has read permission


class ClientProtocol(StreamReaderProtocol):

    def __init__(self, stream_reader: StreamReader, loop: asyncio.AbstractEventLoop):
        super().__init__(stream_reader, loop=loop)


class Client(object):

    def __init__(self):
        self.host = aadb.inner_host
        self.port = aadb.inner_port
        self.reader: StreamReader = None
        self.writer: StreamWriter = None
        self.transport = None

    async def __aenter__(self):
        self.reader = StreamReader(loop=events.get_event_loop())
        protocol = ClientProtocol(self.reader, loop=events.get_event_loop())
        try:
            self.transport, _ = await events.get_event_loop().create_connection(lambda: protocol, self.host, self.port)
            self.writer = StreamWriter(self.transport, protocol, self.reader, events.get_event_loop())
        except ConnectionRefusedError as e:
            print(e)

        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self.writer:
            self.writer.close()
            try:
                await self.writer.wait_closed()
            except AttributeError as e:
                print('close error:', e)

    async def call(self, request: str):
        request_encode = request.encode('utf-8')
        request_length = "{0:04X}".format(len(request_encode)).encode('utf-8')
        self.writer.write(b''.join([request_length, request_encode]))
        await self.writer.drain()
        return await self.receive_done()

    async def receive_done(self):
        status_data = await self.reader.read(4)
        if status_data.decode('utf-8') != Signal.OKAY:
            error = (await self.reader.read(1024)).decode('utf-8')
            print(error)
            raise ConnectionError('write error')

        return True

    async def receive(self):
        head_data = (await self.reader.read(4)).decode('utf-8')
        data_length = int(head_data, 16)
        return (await self.reader.read(data_length)).decode('utf-8')

    async def receive_lines(self, pipeline_func):
        temp_data = b''
        temp_line = ''
        while True:
            receive_data = temp_data + await self.reader.read(1024)
            try:
                result = receive_data.decode('utf-8')
                temp_data = b''
                if not result:
                    if temp_line:
                        pipeline_func(temp_line)
                    break
                result = temp_line + result
                lines = result.split('\n')
                if not result.endswith('\n'):
                    temp_line = lines.pop(-1)
                for line in lines:
                    if not line:
                        continue
                    pipeline_func(line)
            except UnicodeDecodeError:
                temp_data = receive_data

    async def write(self, data):
        self.writer.write(data)



import os
import re
import struct
from typing import List, Dict

import aiofiles

from aadb.transport import Client, Signal, Stats

try:
    from shlex import quote as cmd_quote
except ImportError:
    from pipes import quote as cmd_quote1


class Transport(object):

    def __init__(self):
        self.device = None

    def transport(self, cls):
        def wrapper(*args, **kwargs):
            device = cls(*args, **kwargs)
            self.device = device
            return device

        return wrapper

    def shell(self, **options):
        def decorator(f):
            async def wrapper(*args, **kwargs):
                async with Client() as client:
                    await client.call('host:transport:{}'.format(self.device.serial))
                    cmd_args = '' if not args else args[0]
                    await client.call('shell:{}'.format(options['command']))
                    await client.receive_lines(pipeline_func=kwargs['pipeline_func'])
                return await f(*args, **kwargs)

            return wrapper

        return decorator

    def host(self, **options):
        def decorator(f):
            async def wrapper(*args, **kwargs):
                async with Client() as client:
                    await client.call('host:{}'.format(options['command']))
                    if 'no_receive' not in options:
                        kwargs['parsing_result'](await client.receive())
                return await f(*args, **kwargs)

            return wrapper

        return decorator

    def push(self, **options):
        def decorator(f):
            async def wrapper(*args, **kwargs):
                async with Client() as client:
                    await client.call('host:transport:{}'.format(self.device.serial))
                    # send sync signal
                    await client.call('sync:')
                    # send sync file info data
                    arg = '{dest},{mode}'.format(dest=kwargs['dest_path'],
                                                 mode=options['mode'] | Stats.S_IFREG).encode('utf-8')
                    await client.write(Signal.SEND.encode('utf-8') + struct.pack('<I', len(arg)) + arg)
                    # send data now
                    total_size = os.path.getsize(kwargs['src_path'])
                    async with aiofiles.open(kwargs['src_path'], 'rb') as stream:
                        sent_size = 0
                        while True:
                            chunk = await stream.read(65536)
                            if not chunk:
                                break
                            sent_size += len(chunk)
                            await client.write(Signal.DATA.encode('utf-8') + struct.pack('<I', len(chunk)))
                            await client.write(chunk)
                            if 'progress_func' in options and options['progress_func']:
                                options['progress_func'](total_size, sent_size)
                        await client.write(Signal.DONE.encode('utf-8') + struct.pack('<I', len(chunk)))
                        await client.receive_done()
                return await f(*args, **kwargs)

            return wrapper

        return decorator

    def pull(self, f):
        async def wrapper(*args, **kwargs):
            async with Client() as client:
                await client.call('host:transport:{}'.format(self.device.serial))
                # send sync signal
                await client.call('sync:')
                # send sync file info data
                arg = kwargs['src_path'].encode('utf-8')
                await client.write(Signal.RECV.encode('utf-8') + struct.pack('<I', len(arg)) + arg)
                # send data now
                async with aiofiles.open(kwargs['dest_path'], 'wb') as stream:
                    while True:
                        flag = (await client.reader.read(4)).decode('utf-8')

                        if flag == Signal.DONE:
                            await client.reader.read(4)
                            break

                        data_len = struct.unpack("<I", await client.reader.read(4))[0]
                        data = bytearray()
                        while len(data) < data_len:
                            data += await client.reader.read(data_len - len(data))

                        if flag == Signal.DATA:
                            await stream.write(data)

                        if flag == Signal.FAIL:
                            raise ConnectionError('pull data error: {}'.format(data.decode('utf-8')))

            return await f(*args, **kwargs)

        return wrapper


tp = Transport()


@tp.transport
class Device(object):

    def __init__(self, serial: str):
        self.serial = serial

    async def shell(self, command: str, pipeline=None):

        @tp.shell(command=command)
        async def run_cmd(pipeline_func):
            pass

        def parsing_result(line):
            if pipeline is not None:
                pipeline(line)

        await run_cmd(pipeline_func=parsing_result)

    async def push(self, src: str, dest: str, progress_func=None):
        if not os.path.exists(src):
            raise FileNotFoundError("Can't find the source file {}".format(src))

        @tp.push(progress_func=progress_func, mode=0o644)
        async def push_file(src_path, dest_path):
            pass

        if os.path.isfile(src):
            await push_file(src_path=src, dest_path=dest)
        else:
            for root, dirs, files in os.walk(src):
                root_dir_path = os.path.join(os.path.basename(src), root.replace(src, ''))
                await self.shell('mkdir -p {}/{}'.format(dest, root_dir_path))

                for sub_f in files:
                    await push_file(src_path=os.path.join(root, sub_f),
                                    dest_path=os.path.join(dest, root_dir_path, sub_f))

    async def pull(self, src: str, dest: str):
        @tp.pull
        async def pull_file(src_path, dest_path):
            pass

        await pull_file(src_path=src, dest_path=dest)

    async def list_package(self) -> List[str]:
        pkgs = []

        @tp.shell(command='pm list packages 2>/dev/null')
        async def list_pkgs(pipeline_func):
            pass

        def parsing_line(line):
            m = re.match("^package:(.*?)\r?$", line)
            if m:
                pkgs.append(m.group(1))

        await list_pkgs(pipeline_func=parsing_line)

        return pkgs

    async def get_properties(self) -> Dict[str, str]:
        properties = {}

        @tp.shell(command='getprop')
        async def getprop(pipeline_func):
            pass

        def parsing_line(line):
            m = re.match("^\[([\s\S]*?)\]: \[([\s\S]*?)\]\r?$", line)
            if m:
                properties[m.group(1)] = (m.group(2))

        await getprop(pipeline_func=parsing_line)

        return properties

    async def logcat(self, pipeline):
        @tp.shell(command='logcat')
        async def print_logcat(pipeline_func):
            pass

        await print_logcat(pipeline_func=pipeline)

    @staticmethod
    def __process_install(**kwargs):
        args_dict = {
            'forward_lock': '-l',
            'reinstall': '-r',
            'test': '-t',
            'installer_package_name': '-i',
            'shared_mass_storage': '-s',
            'internal_system_memory': '-f',
            'downgrade': '-d',
            'grand_all_permission': '-g'
        }

        args = []
        for k, v in args_dict.items():
            if k == 'installer_package_name' and kwargs[k]:
                args.append('-i {}'.format(v))
                continue
            if kwargs[k]:
                args.append(v)
        return ' '.join(args)

    async def install(self, path: str,
                      forward_lock: bool = False,
                      reinstall: bool = False,
                      test: bool = False,
                      installer_package_name: str = '',
                      shared_mass_storage: bool = False,
                      internal_system_memory: bool = False,
                      downgrade: bool = False,
                      grand_all_permission: bool = False):
        # push apk to temp first
        result = None
        dest_path = os.path.join('/data/local/tmp', os.path.basename(path))
        await self.push(path, dest_path)

        @tp.shell(command='pm install {} {}'.format(self.__process_install(**(locals())), cmd_quote(dest_path)))
        async def install_apk(pipeline_func):
            pass

        def parsing_result_f(line):
            print(line)
            nonlocal result
            result = re.search("(Success|Failure|Error)\s?(.*)", line)

        await install_apk(pipeline_func=parsing_result_f)

        if result and result.group(1) == "Success":
            return True
        elif result:
            groups = result.groups()
            raise RuntimeError("{} could not be installed - [{}]".format(dest_path, groups[1]))
        else:
            raise RuntimeError("{} could not be installed - [{}]".format(dest_path, result))

    async def is_installed(self, pkg_name: str) -> bool:
        result = False

        def parsing_result(line):
            nonlocal result
            result = 'package:' in line

        await self.shell('pm path {}'.format(pkg_name), pipeline=parsing_result)
        return result

    async def uninstall(self, pkg_name: str) -> bool:
        result = False

        def parsing_result(line):
            nonlocal result
            result = re.search("(Success|Failure.*|.*Unknown package:.*)", line)

        await self.shell('pm uninstall {}'.format(pkg_name), pipeline=parsing_result)
        if result and result.group(1) == "Success":
            return True
        return False



from enum import Enum
from typing import List

from aadb.device import Device
from aadb.device import Transport

tp = Transport()


class AndroidDebugBridge(object):

    class DeviceStatus(Enum):
        OFFLINE = "offline"
        DEVICE = "device"
        BOOTLOADER = "bootloader"

    async def devices(self, state: DeviceStatus = None) -> List[Device]:
        device_list = []

        @tp.host(command='devices')
        async def list_devices(parsing_result):
            pass

        def parsing_result_f(result):
            for line in result.split('\n'):
                if not line:
                    break

                tokens = line.split()
                if state and len(tokens) > 1 and tokens[1] != state:
                    continue

                device_list.append(Device(tokens[0]))

        await list_devices(parsing_result=parsing_result_f)
        return device_list

    async def features(self):
        feature_list = []

        @tp.host(command='features')
        async def list_features(parsing_result):
            pass

        def parsing_result_f(result):
            feature_list.extend(result.split(','))

        await list_features(parsing_result=parsing_result_f)
        return feature_list

    async def version(self):
        ver = 0

        @tp.host(command='version')
        async def check_version(parsing_result):
            pass

        def parsing_result_f(result):
            nonlocal ver
            ver = int(result, 16)

        await check_version(parsing_result=parsing_result_f)
        return ver

    async def kill_server(self):
        @tp.host(command='kill', no_receive=False)
        async def kill_adb_server():
            pass

        await kill_adb_server()

    async def kill_forward_all(self):
        @tp.host(command='killforward-all', no_receive=False)
        async def kill_adb_server():
            pass

        await kill_adb_server()

    async def list_forward(self):
        forwards = {}

        @tp.host(command='list-forward')
        async def list_features(parsing_result):
            pass

        def parsing_result_f(result):
            for line in result.split('\n'):
                if line:
                    serial, local, remote = line.split()
                    if serial not in forwards:
                        forwards[serial] = {}

                    forwards[serial][local] = remote

        await list_features(parsing_result=parsing_result_f)
        return forwards

    async def remote_connect(self, host: str, port: int):
        request_result = False

        @tp.host(command='connect:%s:%d' % (host, port))
        async def connect_remote(parsing_result):
            pass

        def parsing_result_f(result):
            nonlocal request_result
            request_result = 'connected' in result

        await connect_remote(parsing_result=parsing_result_f)
        return request_result

    async def remote_disconnect(self, host: str = None, port: int = None):
        cmd = ''
        if host and port:
            cmd = '{}:{}'.format(host, port)
        if host and not port:
            cmd = host

        @tp.host(command='disconnect:{}'.format(cmd))
        async def disconnect_remote(parsing_result):
            pass

        def parsing_result_f(result):
            print(result)

        await disconnect_remote(parsing_result=parsing_result_f)



from setuptools import setup

setup(
    name='aadb',
    version='1.0.0',
    packages=['aadb'],
    url='',
    license='MIT',
    author='fuxingfu',
    author_email='fxfpro@163.com',
    description='async with android debug bridge'
)
